@@ -1,16 +1,32 @@
-from dataclasses import asdict, dataclass
+import mimetypes
+from dataclasses import asdict, dataclass, field, fields
 from datetime import datetime
 from enum import IntEnum
-from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
+from os import PathLike
+from pathlib import Path
+from typing import IO, Any, Callable, ClassVar, Generic, Literal, Optional, TypeVar, Union
+from typing_extensions import TypeAlias
 
 from .element import Element, transform
+from .parser import Element as RawElement
 from .parser import parse
 
 
+@dataclass
 class ModelBase:
+    __converter__: ClassVar[dict[str, Callable[[Any], Any]]] = {}
+
     @classmethod
     def parse(cls, raw: dict):
-        raise NotImplementedError
+        fs = fields(cls)
+        data = {}
+        for fd in fs:
+            if fd.name in raw:
+                if fd.name in cls.__converter__:
+                    data[fd.name] = cls.__converter__[fd.name](raw[fd.name])
+                else:
+                    data[fd.name] = raw[fd.name]
+        return cls(**data)  # type: ignore
 
     def dump(self) -> dict:
         raise NotImplementedError
@@ -30,11 +46,7 @@ class Channel(ModelBase):
     name: Optional[str] = None
     parent_id: Optional[str] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = raw.copy()
-        data["type"] = ChannelType(raw["type"])
-        return cls(**data)
+    __converter__ = {"type": ChannelType}
 
     def dump(self):
         res = {"id": self.id, "type": self.type.value}
@@ -51,10 +63,6 @@ class Guild(ModelBase):
     name: Optional[str] = None
     avatar: Optional[str] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         res = {"id": self.id}
         if self.name:
@@ -72,12 +80,8 @@ class User(ModelBase):
     avatar: Optional[str] = None
     is_bot: Optional[bool] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
-        res: Dict[str, Any] = {"id": self.id}
+        res: dict[str, Any] = {"id": self.id}
         if self.name:
             res["name"] = self.name
         if self.nick:
@@ -93,25 +97,17 @@ class User(ModelBase):
 class Member(ModelBase):
     user: Optional[User] = None
     nick: Optional[str] = None
-    name: Optional[str] = None
     avatar: Optional[str] = None
     joined_at: Optional[datetime] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = raw.copy()
-        if "user" in raw:
-            data["user"] = User.parse(raw["user"])
-        if "joined_at" in raw:
-            data["joined_at"] = datetime.fromtimestamp(int(raw["joined_at"]) / 1000)
-        return cls(**data)
+    __converter__ = {"user": User.parse, "joined_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000)}
 
     def dump(self):
         res = {}
         if self.user:
             res["user"] = self.user.dump()
-        if self.nick or self.name:
-            res["nick"] = self.nick or self.name
+        if self.nick:
+            res["nick"] = self.nick
         if self.avatar:
             res["avatar"] = self.avatar
         if self.joined_at:
@@ -124,10 +120,6 @@ class Role(ModelBase):
     id: str
     name: Optional[str] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         res = {"id": self.id}
         if self.name:
@@ -149,17 +141,17 @@ class Login(ModelBase):
     user: Optional[User] = None
     self_id: Optional[str] = None
     platform: Optional[str] = None
+    features: list[str] = field(default_factory=list)
+    proxy_urls: list[str] = field(default_factory=list)
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = raw.copy()
-        if "user" in raw:
-            data["user"] = User(**raw["user"])
-        data["status"] = LoginStatus(data["status"])
-        return cls(**data)
+    __converter__ = {"user": User.parse, "status": LoginStatus}
 
     def dump(self):
-        res: Dict[str, Any] = {"status": self.status.value}
+        res: dict[str, Any] = {
+            "status": self.status.value,
+            "features": self.features,
+            "proxy_urls": self.proxy_urls,
+        }
         if self.user:
             res["user"] = self.user.dump()
         if self.self_id:
@@ -168,6 +160,39 @@ class Login(ModelBase):
             res["platform"] = self.platform
         return res
 
+    @property
+    def id(self) -> Optional[str]:
+        return self.self_id or (self.user.id if self.user else None)
+
+
+@dataclass
+class LoginPreview(ModelBase):
+    user: User
+    platform: str
+    status: Optional[LoginStatus] = None
+    features: list[str] = field(default_factory=list)
+    proxy_urls: list[str] = field(default_factory=list)
+
+    __converter__ = {"user": User.parse, "status": LoginStatus}
+
+    def dump(self):
+        res: dict[str, Any] = {
+            "user": self.user.dump(),
+            "platform": self.platform,
+            "features": self.features,
+            "proxy_urls": self.proxy_urls,
+        }
+        if self.status:
+            res["status"] = self.status.value
+        return res
+
+    @property
+    def id(self) -> str:
+        return self.user.id
+
+
+LoginType = Union[Login, LoginPreview]
+
 
 @dataclass
 class ArgvInteraction(ModelBase):
@@ -175,10 +200,6 @@ class ArgvInteraction(ModelBase):
     arguments: list
     options: Any
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         return asdict(self)
 
@@ -187,10 +208,6 @@ class ArgvInteraction(ModelBase):
 class ButtonInteraction(ModelBase):
     id: str
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         return asdict(self)
 
@@ -211,7 +228,7 @@ class Identify(ModelBase):
 
 @dataclass
 class Ready(ModelBase):
-    logins: List[Login]
+    logins: list[Login]
 
 
 @dataclass
@@ -229,7 +246,7 @@ class MessageObject(ModelBase):
     def from_elements(
         cls,
         id: str,
-        content: List[Element],
+        content: list[Element],
         channel: Optional[Channel] = None,
         guild: Optional[Guild] = None,
         member: Optional[Member] = None,
@@ -240,31 +257,31 @@ class MessageObject(ModelBase):
         return cls(id, "".join(str(i) for i in content), channel, guild, member, user, created_at, updated_at)
 
     @property
-    def message(self) -> List[Element]:
+    def message(self) -> list[Element]:
         return transform(parse(self.content))
 
+    @message.setter
+    def message(self, value: list[Element]):
+        self.content = "".join(str(i) for i in value)
+
     @classmethod
     def parse(cls, raw: dict):
-        data = {
-            "id": raw["id"],
-            "content": raw["content"],
-        }
-        if "channel" in raw:
-            data["channel"] = Channel.parse(raw["channel"])
-        if "guild" in raw:
-            data["guild"] = Guild.parse(raw["guild"])
-        if "member" in raw:
-            data["member"] = Member.parse(raw["member"])
-        if "user" in raw:
-            data["user"] = User.parse(raw["user"])
-        if "created_at" in raw:
-            data["created_at"] = datetime.fromtimestamp(int(raw["created_at"]) / 1000)
-        if "updated_at" in raw:
-            data["updated_at"] = datetime.fromtimestamp(int(raw["updated_at"]) / 1000)
-        return cls(**data)
+        if "elements" in raw and "content" not in raw:
+            content = [RawElement(*item.values()) for item in raw["elements"]]
+            raw["content"] = "".join(str(i) for i in content)
+        return super().parse(raw)
+
+    __converter__ = {
+        "channel": Channel.parse,
+        "guild": Guild.parse,
+        "member": Member.parse,
+        "user": User.parse,
+        "created_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
+        "updated_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
+    }
 
     def dump(self):
-        res: Dict[str, Any] = {"id": self.id, "content": self.content}
+        res: dict[str, Any] = {"id": self.id, "content": self.content}
         if self.channel:
             res["channel"] = self.channel.dump()
         if self.guild:
@@ -281,17 +298,52 @@ class MessageObject(ModelBase):
 
 
 @dataclass
-class Event:
+class MessageReceipt(ModelBase):
+    id: str
+    content: Optional[str] = None
+
+    @classmethod
+    def from_elements(
+        cls,
+        id: str,
+        content: Optional[list[Element]] = None,
+    ):
+        return cls(id, "".join(str(i) for i in content) if content else None)
+
+    @property
+    def message(self) -> Optional[list[Element]]:
+        return transform(parse(self.content)) if self.content else None
+
+    @message.setter
+    def message(self, value: Optional[list[Element]]):
+        self.content = "".join(str(i) for i in value) if value else None
+
+    @classmethod
+    def parse(cls, raw: dict):
+        if "elements" in raw and "content" not in raw:
+            content = [RawElement(*item.values()) for item in raw["elements"]]
+            raw["content"] = "".join(str(i) for i in content)
+        return super().parse(raw)
+
+    def dump(self):
+        res = {"id": self.id}
+        if self.content:
+            res["content"] = self.content
+        return res
+
+
+@dataclass
+class Event(ModelBase):
     id: int
     type: str
-    platform: str
-    self_id: str
     timestamp: datetime
+    platform: Optional[str] = None
+    self_id: Optional[str] = None
     argv: Optional[ArgvInteraction] = None
     button: Optional[ButtonInteraction] = None
     channel: Optional[Channel] = None
     guild: Optional[Guild] = None
-    login: Optional[Login] = None
+    login: Optional[LoginType] = None
     member: Optional[Member] = None
     message: Optional[MessageObject] = None
     operator: Optional[User] = None
@@ -301,47 +353,52 @@ class Event:
     _type: Optional[str] = None
     _data: Optional[dict] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = {
-            "id": raw["id"],
-            "type": raw["type"],
-            "platform": raw["platform"],
-            "self_id": raw["self_id"],
-            "timestamp": datetime.fromtimestamp(int(raw["timestamp"]) / 1000),
-        }
-        if "argv" in raw:
-            data["argv"] = ArgvInteraction.parse(raw["argv"])
-        if "button" in raw:
-            data["button"] = ButtonInteraction.parse(raw["button"])
-        if "channel" in raw:
-            data["channel"] = Channel.parse(raw["channel"])
-        if "guild" in raw:
-            data["guild"] = Guild.parse(raw["guild"])
-        if "login" in raw:
-            data["login"] = Login.parse(raw["login"])
-        if "member" in raw:
-            data["member"] = Member.parse(raw["member"])
-        if "message" in raw:
-            data["message"] = MessageObject.parse(raw["message"])
-        if "operator" in raw:
-            data["operator"] = User.parse(raw["operator"])
-        if "role" in raw:
-            data["role"] = Role.parse(raw["role"])
-        if "user" in raw:
-            data["user"] = User.parse(raw["user"])
-        if "_type" in raw:
-            data["_type"] = raw["_type"]
-        if "_data" in raw:
-            data["_data"] = raw["_data"]
-        return cls(**data)
+    __converter__ = {
+        "timestamp": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
+        "argv": ArgvInteraction.parse,
+        "button": ButtonInteraction.parse,
+        "channel": Channel.parse,
+        "guild": Guild.parse,
+        "login": lambda raw: (
+            LoginPreview.parse(
+                raw if raw["user"] else {**raw, "user": {"id": raw["self_id"]}} if "self_id" in raw else raw
+            )
+            if "user" in raw
+            else Login.parse(raw)
+        ),
+        "member": Member.parse,
+        "message": MessageObject.parse,
+        "operator": User.parse,
+        "role": Role.parse,
+        "user": User.parse,
+    }
+
+    @property
+    def platform_(self):
+        if self.platform:
+            return self.platform
+        if self.login and self.login.platform:
+            return self.login.platform
+        raise ValueError("platform not found")
+
+    @property
+    def self_id_(self):
+        if self.self_id:
+            return self.self_id
+        if self.login and self.login.id:
+            return self.login.id
+        raise ValueError("self_id not found")
+
+    def __post_init__(self):
+        _ = self.platform_
+        _ = self.self_id_
 
     def dump(self):
         res = {
             "id": self.id,
             "type": self.type,
-            "platform": self.platform,
-            "self_id": self.self_id,
+            "platform": self.platform_,
+            "self_id": self.self_id_,
             "timestamp": int(self.timestamp.timestamp() * 1000),
         }
         if self.argv:
@@ -376,16 +433,58 @@ T = TypeVar("T", bound=ModelBase)
 
 @dataclass
 class PageResult(ModelBase, Generic[T]):
-    data: List[T]
+    data: list[T]
     next: Optional[str] = None
 
     @classmethod
-    def parse(cls, raw: dict, parser: Callable[[dict], T]) -> "PageResult[T]":
-        data = [parser(item) for item in raw["data"]]
-        return cls(data, raw.get("next"))
+    def parse(cls, raw: dict, parser: Optional[Callable[[dict], T]] = None) -> "PageResult[T]":
+        data = [(parser or ModelBase.parse)(item) for item in raw["data"]]
+        return cls(data, raw.get("next"))  # type: ignore
 
     def dump(self):
         res: dict = {"data": [item.dump() for item in self.data]}
         if self.next:
             res["next"] = self.next
         return res
+
+
+@dataclass
+class PageDequeResult(PageResult[T]):
+    prev: Optional[str] = None
+
+    @classmethod
+    def parse(cls, raw: dict, parser: Optional[Callable[[dict], T]] = None) -> "PageDequeResult[T]":
+        data = [(parser or ModelBase.parse)(item) for item in raw["data"]]
+        return cls(data, raw.get("next"), raw.get("prev"))  # type: ignore
+
+    def dump(self):
+        res: dict = {"data": [item.dump() for item in self.data]}
+        if self.next:
+            res["next"] = self.next
+        if self.prev:
+            res["prev"] = self.prev
+        return res
+
+
+Direction: TypeAlias = Literal["before", "after", "around"]
+Order: TypeAlias = Literal["asc", "desc"]
+
+
+@dataclass
+class Upload:
+    file: Union[bytes, IO[bytes], PathLike]
+    mimetype: str = "image/png"
+    name: Optional[str] = None
+
+    def __post_init__(self):
+        if isinstance(self.file, PathLike):
+            self.mimetype = mimetypes.guess_type(str(self.file))[0] or self.mimetype
+            self.name = Path(self.file).name
+
+    def dump(self):
+        file = self.file
+
+        if isinstance(file, PathLike):
+            file = open(file, "rb")
+
+        return {"value": file, "filename": self.name, "content_type": self.mimetype}
